#!/usr/bin/perl
use strict;
use warnings;

use Getopt::Long;
use English;

use File::Spec;

$ENV{PATH} = '/bin:/usr/bin';

my $usage = "usage: $0 --ssl-dir=<ssl_directory> [ --out-file=<out_file> ] [ --help ]\n";

my $ssl_dir = '';
my $out_file = '';
my $help = '';

GetOptions("ssl-dir=s" => \$ssl_dir, "out-file:s" => \$out_file, "help" => \$help);

if ($help or not $ssl_dir) {
  die $usage;
}

unless (-d $ssl_dir) {
  die "$ssl_dir is not a directory";
}

my @files_to_cat = (File::Spec->catfile($ssl_dir, 'server.crt'),
		    File::Spec->catfile($ssl_dir, 'server.key'));

my @content;

foreach my $file (@files_to_cat) {

  unless (-r $file) {
    die "Could not read ${file}.";
  }

  open(SSL_FILE, $file) or die "Could not open '$file' for reading: $OS_ERROR";

  my @lines = <SSL_FILE>;
  push @content, @lines;
}

close(SSL_FILE);

if ($out_file) {
  open(PEM_FILE, ">$out_file") or die "Could not open '$out_file' for writing: $OS_ERROR";

  print PEM_FILE @content;

  close(PEM_FILE);

  chmod(0600, $out_file); # rw by current user (probably root) only

  my ($uid, $gid) = lookup_user_ids('jabberd');
  chown($uid, $gid, $out_file);
}
else {
  print @content;
}

exit 0;

sub lookup_user_ids {
  my $username = shift;

  my ($login, $pass, $uid, $gid) = getpwnam($username);

  return ($uid, $gid);
}
