#!/usr/bin/perl

# Copyright (c) 2002-2004 Red Hat, Inc.
# All rights reserved.
#
# $Id: validate-sat-cert.pl,v 1.48 2005-01-14 23:05:47 cturner Exp $

use strict;
use lib '/var/www/lib';

use RHN::SatelliteCert;

use Getopt::Long;

my $keyring = "/etc/webapp-keyring.gpg";
my $quiet;
my $required_version;
my $show_help;
my $nosig;
my $allow_old;

GetOptions("keyring=s", \$keyring, "quiet" => \$quiet, "nosig" => \$nosig, "allow-old" => \$allow_old,
           "required-version=s" => \$required_version, "help" => \$show_help);

if ($show_help) {
  print "See 'man validate-sat-cert' for details on usage.\n";
  exit 1;
}

my $data = join("", <>);

my ($signature, $cert) = RHN::SatelliteCert->parse_cert($data);

if (not $cert->get_field('generation') and not $allow_old) {
  print "Error: Your satellite certificate is no longer valid.  Please contact your support representative.\n";
  exit 3;
}

if ($required_version and $required_version ne $cert->version) {
  printf "Error: certificate and satellite version mismatch ('%s' vs '%s')\n",
	      $required_version, $cert->version;
  exit 2;
}

$cert->check_required_fields();

if ($nosig) {
  print "GPG signature *not* checked.\nCertificate parsed correctly.\n"
    unless $quiet;
  exit 0;
}

my $result = $cert->check_signature($signature, $keyring);

print "GPG signature signed by $result->{user} ($result->{keyid}).\n"
  unless $quiet;


